﻿/*
Vampire Attack 2
Copyright (c) 2009-2010 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.vampireattack2
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;

	public class Lich
	extends MovieClip
	{
// ***** CONSTANTS **********************************************************

		private static const MIN_DISTANCE_FOR_MOVEMENT:int = 700 * 700;
		private static const DEATH_DISTANCE_SQUARED: int = 20 * 20;
		private static const STARTING_HEALTH:int = 3;
		private static const DIAMETER:int = 15;
		private static const FIRE_RATE:Number = 5;

// ***** VARIABLES **********************************************************

		public var _health:int;
		public var _lastLaunch:Number;
		
// ***** CONSTRUCTOR ********************************************************

		public function Lich()
		{
			_health = STARTING_HEALTH;
			_lastLaunch = 0;
		}


// ***** Functions - Event handlers *****************************************


// ***** Functions - General control ****************************************

		public function killIt():int
		{
			if ( _health <= 0 )
				return 0;
			var rv:int = 0;
			
			--_health;
			if (_health < 1) {
				gotoAndPlay("Death");
				rv = 2;
			} else if (_health == 2)
				gotoAndPlay("Wounded");
			else
				gotoAndPlay("Critical");

			return rv;
		}
		
		public function isAlive()
		{
			return (_health > 0);
		}

		/* returns true if bat hits player */
		public function updateState(map:Map, timeDelta:Number):Boolean
		{
			if ( _health < 1 ) return false;

			_lastLaunch += timeDelta;
			
			// distance from player
			var deltaX = map._player.x - x;
			var deltaY = map._player.y - y;
			var distanceSquared:Number = deltaX * deltaX + deltaY * deltaY;
			if (distanceSquared > MIN_DISTANCE_FOR_MOVEMENT) return false;
			if (distanceSquared < DEATH_DISTANCE_SQUARED) return true;
			var distanceFromPlayer:int = Math.sqrt(distanceSquared) / 32;
			
			// figure out movement
			var ang = Math.floor((map.findAngle(map._player.x, map._player.y, x, y) + 90));
			
			// see if player in line-of-sight
			var playerSeen:Boolean = true;
			var stepX:Number = Map.TILE_SIZE * -Math.sin(ang * Map.RADS_PER_DEG);
			var stepY:Number = Map.TILE_SIZE * Math.cos(ang * Map.RADS_PER_DEG);
			for (var cntr:int = 0; cntr < distanceFromPlayer; ++cntr)
				if (map.getTile((x + stepX * cntr)/Map.TILE_SIZE, (y + stepY * cntr)/Map.TILE_SIZE) > 15)
					playerSeen = false;
			
			// if can see player, perform movement
			if (playerSeen) {
				rotation = ang;
				if (_lastLaunch > FIRE_RATE) {
					map.launchLightning(x, y, ang-180);
					_lastLaunch = 0;
				}
			}
			return false;
		}
	} // end class
} // end package
