﻿/*
Vampire Attack 2
Copyright (c) 2009-2010 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.vampireattack2
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.media.Sound;

	public class Map
	extends MovieClip
	{
// ***** CONSTANTS **********************************************************
		
		public static const RADS_PER_DEG = Math.PI / 180;
		public static const TILE_SIZE:int = 32;
		public const MAP_WIDTH:int = 64;
		public const MAP_HEIGHT:int = 64;
		
		public const BASE64ENC:String = "-+abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
		
		public static const DIR_NORTH:int = 0;
		public static const DIR_EAST:int = 1;
		public static const DIR_SOUTH:int = 2;
		public static const DIR_WEST:int = 3;
		public const PLAYER_SIZE:int = 20;
		public const PLAYER_MOVESPEED:int = 2;

		public static const GAME_LOST:int = -1;
		public static const GAME_PLAYING:int = 0;
		public static const GAME_WON:int = 1;
		
// ***** VARIABLES **********************************************************

		private var _tiles:Array;
		public var _player:Player;
		public var _loadedStake:Stake;
		public var _stakes:Array;
		public var _monsters:Array;
		public var _lightnings:Array;
		
		public var _winTileX:int = -1;
		public var _winTileY:int = -1;
		public var _score:int;

		public var _launchSound:Sound;
		public var _hitSound:Sound;
		
// ***** CONSTRUCTOR ********************************************************

		public function Map()
		{
			var cntrX:int;
			var cntrY:int;
			var indx:int = 0;
			
			_tiles = new Array();
			_stakes = new Array();
			_lightnings = new Array();
			_monsters = new Array();
			for (cntrY = 0; cntrY < MAP_HEIGHT; ++cntrY)
				for (cntrX = 0; cntrX < MAP_WIDTH; ++cntrX) {
					_tiles[indx] = new Tile();
					_tiles[indx].x = cntrX * TILE_SIZE;
					_tiles[indx].y = cntrY * TILE_SIZE;
					addChild(_tiles[indx]);
					++indx;
				}
				
			_player = new Player(this);
			_player.x = 1.5 * TILE_SIZE;
			_player.y = 61.5 * TILE_SIZE;
			addChild(_player);
			_player.doneLoad();
			_launchSound = new launch();
			_hitSound = new hit();
		}


// ***** Functions - Event handlers *****************************************


// ***** Functions - General control ****************************************

		public function findAngle(x1, y1, x2, y2)
		{
			var distX = x2 - x1;
			var distY = y2 - y1;
			var absX = Math.abs(distX);
			var absY = Math.abs(distY);
			if (absX == 0)
			{
				if (distY > 0)
					return 90.0;
				else
					return -90.0;
			}
			var tan = absY / absX;
			var atan = Math.atan(tan);
			var deg = atan / Math.PI * 180.0;
			if (distX < 0)
			{
				if (distY < 0)
				{
					return (-180.0 + deg);
				}
				else
				{
					return (180.0 - deg);
				}
			}
			else
			{
				if (distY < 0)
				{
					return (-deg);
				}
				else
				{
					return (deg);
				}
			}
		}

		public function setLevelData(xml:XML, newGame:Boolean = true)
		{
			var indx:int = 0;
			var cntr:int;
			var id:int;
			var s:String = xml.MAP;
			var temp:MovieClip;

			if (newGame)
				_score = 0;
			while (_stakes.length > 0) {
				temp = _stakes.pop();
				removeChild(temp);
			}
			while (_monsters.length > 0) {
				temp = _monsters.pop();
				removeChild(temp);
			}
			
			for (cntr = 0; cntr < s.length; ++cntr) {
				id = BASE64ENC.indexOf(s.charAt(cntr));
				if (id >= 0) {
					_tiles[indx].setTile(id);
					if ((id == LevelData.TILE_DIRT_BATSPAWN) || (id == LevelData.TILE_BRICK_BATSPAWN)) {
						var bat:Bat = new Bat();
						bat.x = _tiles[indx].x + TILE_SIZE / 2;
						bat.y = _tiles[indx].y + TILE_SIZE / 2;
						addChild(bat);
						_monsters.push(bat);
					}
					if ( (id == LevelData.TILE_BRICK_SKELETONSPAWN) || (id == LevelData.TILE_ROCK_SKELETONSPAWN)) {
						var skeleton:Skeleton = new Skeleton();
						skeleton.x = _tiles[indx].x + TILE_SIZE / 2;
						skeleton.y = _tiles[indx].y + TILE_SIZE / 2;
						addChild(skeleton);
						_monsters.push(skeleton);
					}
					if (id == LevelData.TILE_ROCK_LICHSPAWN) {
						var lich:Lich = new Lich();
						lich.x = _tiles[indx].x + TILE_SIZE / 2;
						lich.y = _tiles[indx].y + TILE_SIZE / 2;
						addChild(lich);
						_monsters.push(lich);
					}
					++indx;
				}
			}
			_player.x = 1.5 * TILE_SIZE;
			_player.y = 61.5 * TILE_SIZE;
			_winTileX = xml.WIN.@tileX;
			_winTileY = xml.WIN.@tileY;
//			trace ("Goal location for episode is " +_winTileX+","+_winTileY);
		}
		
		public function doneLoad()
		{
			_loadedStake = new Stake();
			addChild(_loadedStake);
		}
		
		public function fireStake():void
		{
//			trace ("Should be launching stake");
			if (_loadedStake != null) { // make sure we are ready to fire
				_stakes.push(_loadedStake);
				_loadedStake.launch();
				_player.launch();
				_loadedStake = null;
				// todo - launch sound support
				_launchSound.play();
			}
		}

		public function launchLightning(px:Number, py:Number, ang:Number):void
		{
//			trace ("Should be launching lightning from " +px + ","+py+" angle " + ang);
			var lightning:Lightning = new Lightning(px, py, ang);
			addChild(lightning);
			_lightnings.push(lightning);
		}

		public function movePlayer(dir:int):void
		{
			var targetX:int = _player.x;
			var targetY:int = _player.y;
			var targetTileX:int = _player.x / TILE_SIZE;
			var targetTileY:int = _player.y / TILE_SIZE;

			switch (dir) {
				case DIR_NORTH:
					targetY -= PLAYER_MOVESPEED;
					targetTileY = (_player.y - PLAYER_SIZE - PLAYER_MOVESPEED) / TILE_SIZE;
					break;
				case DIR_EAST:
					targetX += PLAYER_MOVESPEED;
					targetTileX = (_player.x + PLAYER_SIZE + PLAYER_MOVESPEED) / TILE_SIZE;
					break;
				case DIR_SOUTH:
					targetY += PLAYER_MOVESPEED;
					targetTileY = (_player.y + PLAYER_SIZE + PLAYER_MOVESPEED) / TILE_SIZE;
					break;
				case DIR_WEST:
					targetX -= PLAYER_MOVESPEED;
					targetTileX = (_player.x - PLAYER_SIZE - PLAYER_MOVESPEED) / TILE_SIZE;
					break;
			}
			var tileIndex:int = targetTileY * MAP_WIDTH + targetTileX;
			if (_tiles[tileIndex].getTile() < 16) {
				_player.x = targetX;
				_player.y = targetY;
			}
		}
		
		public function getTile(tx:int, ty:int):int
		{
			return _tiles[(ty & 63)* MAP_WIDTH + (tx & 63)].getTile();
		}

		public function canEnter(pixelX:Number, pixelY:Number, diameter:Number, self:MovieClip = null, canFly:Boolean = false):Boolean
		{
			var tx1:int = (pixelX - diameter) / TILE_SIZE;
			var ty1:int = (pixelY - diameter) / TILE_SIZE;
			var tx2:int = (pixelX + diameter) / TILE_SIZE;
			var ty2:int = (pixelY + diameter) / TILE_SIZE;
			
			var tileToCheck:int;
			var enterable:Boolean = true;
			for (var cntrX:int = tx1; cntrX <= tx2; ++cntrX)
				for (var cntrY:int = ty1; cntrY <= ty2; ++cntrY) {
					tileToCheck = getTile(cntrX, cntrY);
					if (tileToCheck > 23)
						enterable = false;
					else if ((!canFly) && (tileToCheck > 15))
						enterable = false;
				}
				
			if ((enterable) && (self != null)) {
				var dx:Number;
				var dy:Number;
				var hitDist:Number = diameter * diameter;
				var m:MovieClip;
				for (var cntr:int = 0; cntr < _monsters.length; ++cntr) {
					if (_monsters[cntr].isAlive() == false)
						continue;
					m = _monsters[cntr];
					if (m == self)
						continue;
					dx = (pixelX - m.x) * (pixelX - m.x);
					dy = (pixelY - m.y) * (pixelY - m.y);
					if ((dx+dy) < hitDist)
						enterable = false;
				}
			}
			return enterable;					
		}
		
		public function updateWorld(timeDelta:Number)
		{
			var cntr:int;
			var cntrMonster:int;
			
			// step 1 - move the monsters
			for (cntr = 0; cntr < _monsters.length; ++cntr)
				if (_monsters[cntr].updateState(this, timeDelta))
					return GAME_LOST;
				
			// step 2 - move the stakes
			if (_loadedStake != null)
				_loadedStake.updateState(this, timeDelta);
			for (cntr = 0; cntr < _stakes.length; ++cntr) {
				_stakes[cntr].updateState(this, timeDelta);
				if ( ! _stakes[cntr]._isAlive)
					continue;
				for (cntrMonster = 0; cntrMonster < _monsters.length; ++cntrMonster)
					if ((_stakes[cntr]._isAlive) && (_monsters[cntrMonster].isAlive()) && (_monsters[cntrMonster].hitTestObject(_stakes[cntr]))) {
						_stakes[cntr]._isAlive = false;
						_score += _monsters[cntrMonster].killIt();
						_hitSound.play();
					}

			}
			if (_stakes.length > 0)
				if (_stakes[0].isAlive() == false) {
					removeChild(_stakes[0]);
					_stakes.shift();
				}

			// step 3 - move the lightning balls
			for (cntr = 0; cntr < _lightnings.length; ++cntr) {
				if (_lightnings[cntr].updateState(this, timeDelta))
					return GAME_LOST;
				for (cntrMonster = 0; cntrMonster < _stakes.length; ++cntrMonster)
					if ((_stakes[cntrMonster]._isAlive) && (_stakes[cntrMonster].hitTestObject(_lightnings[cntr]))) {
						_stakes[cntrMonster]._isAlive = false;
						_lightnings[cntr].killIt();
					}
			}
			if (_lightnings.length > 0)
				if (_lightnings[0].isAlive() == false) {
					removeChild(_lightnings[0]);
					_lightnings.shift();
				}

			// step 4 - verify game state (win, lose, still playing)
			if ( (_player.y < 52) && (_player.x > (_winTileX*TILE_SIZE)) && (_player.x < (_winTileX*TILE_SIZE+TILE_SIZE)) )
				return GAME_WON;
			var tileToCheck:int = getTile(_player.x / 32, _player.y / 32);
			if ( (tileToCheck == 14) || (tileToCheck == 15) )
				return GAME_WON;
			return GAME_PLAYING;
		}

		public function getScore():int
		{
			return _score;
		}
	} // end class
} // end package

